from pydantic import validate_call
from vonage_http_client.http_client import HttpClient

from .models import BaseMessage
from .responses import SendMessageResponse


class Messages:
    """Calls Vonage's Messages API.

    This class provides methods to interact with Vonage's Messages API, allowing you to send messages.

    Args:
        http_client (HttpClient): An instance of the HttpClient class used to make HTTP requests.
    """

    def __init__(self, http_client: HttpClient) -> None:
        self._http_client = http_client

    @property
    def http_client(self) -> HttpClient:
        """The HTTP client used to make requests to the Messages API.

        Returns:
            HttpClient: The HTTP client used to make requests to the Messages API.
        """
        return self._http_client

    @validate_call
    def send(self, message: BaseMessage) -> SendMessageResponse:
        """Send a message using Vonage's Messages API.

        Args:
            message (BaseMessage): The message to be sent as a Pydantic model.
                Use the provided models (in `vonage_messages.models`) to create messages and pass them in to this method.

        Returns:
            SendMessageResponse: Response model containing the unique identifier of the sent message.
                Access the identifier with the `message_uuid` attribute.
        """
        response = self._http_client.post(
            self._http_client.api_host,
            '/v1/messages',
            message.model_dump(by_alias=True, exclude_none=True) or message,
        )
        return SendMessageResponse(**response)



from typing import Optional

from pydantic import BaseModel, Field
from vonage_utils.types import PhoneNumber

from .enums import WebhookVersion


class BaseMessage(BaseModel):
    to: PhoneNumber
    client_ref: Optional[str] = Field(None, max_length=100)
    webhook_url: Optional[str] = None
    webhook_version: Optional[WebhookVersion] = None



from typing import Literal, Optional

from pydantic import BaseModel, Field, model_validator

from .base_message import BaseMessage
from .enums import ChannelType, MessageType


class MessengerResource(BaseModel):
    url: str


class MessengerOptions(BaseModel):
    category: Optional[Literal['response', 'update', 'message_tag']] = None
    tag: Optional[str] = None

    @model_validator(mode='after')
    def check_tag_if_category_message_tag(self):
        if self.category == 'message_tag' and not self.tag:
            raise ValueError('"tag" is required when "category" == "message_tag"')
        return self


class BaseMessenger(BaseMessage):
    to: str = Field(..., min_length=1, max_length=50)
    from_: str = Field(..., min_length=1, max_length=50, serialization_alias='from')
    messenger: Optional[MessengerOptions] = None
    channel: ChannelType = ChannelType.MESSENGER


class MessengerText(BaseMessenger):
    text: str = Field(..., max_length=640)
    message_type: MessageType = MessageType.TEXT


class MessengerImage(BaseMessenger):
    image: MessengerResource
    message_type: MessageType = MessageType.IMAGE


class MessengerAudio(BaseMessenger):
    audio: MessengerResource
    message_type: MessageType = MessageType.AUDIO


class MessengerVideo(BaseMessenger):
    video: MessengerResource
    message_type: MessageType = MessageType.VIDEO


class MessengerFile(BaseMessenger):
    file: MessengerResource
    message_type: MessageType = MessageType.FILE



from typing import List, Literal, Optional, Union

from pydantic import BaseModel, ConfigDict, Field
from vonage_utils.types import PhoneNumber

from .base_message import BaseMessage
from .enums import ChannelType, MessageType


class WhatsappContext(BaseModel):
    message_uuid: str


class BaseWhatsapp(BaseMessage):
    from_: Union[PhoneNumber, str] = Field(..., serialization_alias='from')
    context: Optional[WhatsappContext] = None
    channel: ChannelType = ChannelType.WHATSAPP


class WhatsappText(BaseWhatsapp):
    text: str = Field(..., max_length=4096)
    message_type: MessageType = MessageType.TEXT


class WhatsappImageResource(BaseModel):
    url: str
    caption: Optional[str] = Field(None, min_length=1, max_length=3000)


class WhatsappImage(BaseWhatsapp):
    image: WhatsappImageResource
    message_type: MessageType = MessageType.IMAGE


class WhatsappAudioResource(BaseModel):
    url: str = Field(..., min_length=10, max_length=2000)


class WhatsappAudio(BaseWhatsapp):
    audio: WhatsappAudioResource
    message_type: MessageType = MessageType.AUDIO


class WhatsappVideoResource(BaseModel):
    url: str
    caption: Optional[str] = None


class WhatsappVideo(BaseWhatsapp):
    video: WhatsappVideoResource
    message_type: MessageType = MessageType.VIDEO


class WhatsappFileResource(BaseModel):
    url: str
    caption: Optional[str] = None
    name: Optional[str] = None


class WhatsappFile(BaseWhatsapp):
    file: WhatsappFileResource
    message_type: MessageType = MessageType.FILE


class WhatsappTemplateResource(BaseModel):
    name: str
    parameters: Optional[List[str]] = None

    model_config = ConfigDict(extra='allow')


class WhatsappTemplateSettings(BaseModel):
    locale: Optional[str] = 'en_US'
    policy: Optional[Literal['deterministic']] = None


class WhatsappTemplate(BaseWhatsapp):
    template: WhatsappTemplateResource
    whatsapp: WhatsappTemplateSettings = WhatsappTemplateSettings()
    message_type: MessageType = MessageType.TEMPLATE


class WhatsappStickerUrl(BaseModel):
    url: str


class WhatsappStickerId(BaseModel):
    id: str


class WhatsappSticker(BaseWhatsapp):
    sticker: Union[WhatsappStickerUrl, WhatsappStickerId]
    message_type: MessageType = MessageType.STICKER


class WhatsappCustom(BaseWhatsapp):
    custom: Optional[dict] = None
    message_type: MessageType = MessageType.CUSTOM
